#ifndef GISH_GAME_ENGLISH_H
#define GISH_GAME_ENGLISH_H
/*
Copyright (C) 2005, 2010 - Cryptic Sea

This file is part of Gish.

Gish is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//logic.c
#define TXT_TOUCHDOWN_GRAY 							"Touchdown Gray!"
#define TXT_FIELDGOAL_GRAY							"Field Goal Gray!"
#define TXT_TOUCHDOWN_GISH							"Touchdown Gish!"
#define TXT_FIELDGOAL_GISH							"Field Goal Gish!"
#define TXT_GRAYWINS_ROUND							"Gray Wins Round!"
#define TXT_GISHWINS_ROUND							"Gish Wins Round!"
#define TXT_COMBO												"Combo"

#define TXT_DIALOG2_LINE1								"Hey there, look"
#define TXT_DIALOG2_LINE2								"what the cat"
#define TXT_DIALOG2_LINE3								"flushed down, its"
#define TXT_DIALOG2_LINE4								"our old buddy"
#define TXT_DIALOG2_LINE5								"Gish... whaddya"
#define TXT_DIALOG2_LINE6								"say boys, lets"
#define TXT_DIALOG2_LINE7								"show old Gishy"
#define TXT_DIALOG2_LINE8								"here a good"
#define TXT_DIALOG2_LINE9								"time...HONEY"
#define TXT_DIALOG2_LINE10							"BUCKET STYLE!"

#define TXT_DIALOG3_LINE1								"Well well well,"
#define TXT_DIALOG3_LINE2								"it be one o dem"
#define TXT_DIALOG3_LINE3								"blobby tings..."
#define TXT_DIALOG3_LINE4								"HAR HAR HAR dis"
#define TXT_DIALOG3_LINE5								"one won't be"
#define TXT_DIALOG3_LINE6								"gettin away so"
#define TXT_DIALOG3_LINE7								"easy naw.  Why"
#define TXT_DIALOG3_LINE8								"you be lookin"
#define TXT_DIALOG3_LINE9								"so worried dere?"

#define TXT_DIALOG4_LINE1								"I be jus lookin"
#define TXT_DIALOG4_LINE2								"to talk s'all..."
#define TXT_DIALOG4_LINE3								"HAR HAR HAR,"
#define TXT_DIALOG4_LINE4								"good ol Paunchy"
#define TXT_DIALOG4_LINE5								"gunna be cookin"
#define TXT_DIALOG4_LINE6								"up some"
#define TXT_DIALOG4_LINE7								"tar-iffic gumbo"
#define TXT_DIALOG4_LINE8								"dis night, I"
#define TXT_DIALOG4_LINE9								"garontee"

#define TXT_DIALOG5_LINE1								"Oh, look who it"
#define TXT_DIALOG5_LINE2								"is sister, its"
#define TXT_DIALOG5_LINE3								"the Gish..Yess,"
#define TXT_DIALOG5_LINE4								"Gish..We have"
#define TXT_DIALOG5_LINE5								"been waiting for"
#define TXT_DIALOG5_LINE6								"you little one,"
#define TXT_DIALOG5_LINE7								"you have many"
#define TXT_DIALOG5_LINE8								"'determined'"
#define TXT_DIALOG5_LINE9								"admirers..Yess,"
#define TXT_DIALOG5_LINE10							"determined.."

#define TXT_DIALOG6_LINE1								"Your little"
#define TXT_DIALOG6_LINE2								"adventure ends"
#define TXT_DIALOG6_LINE3								"here, there is"
#define TXT_DIALOG6_LINE4								"no happy ending"
#define TXT_DIALOG6_LINE5								"for you now..."
#define TXT_DIALOG6_LINE6								"Yess no happy"
#define TXT_DIALOG6_LINE7								"for you!"

#define TXT_DIALOG7_LINE1								"RRRAAAWLL, YOU"
#define TXT_DIALOG7_LINE2								"AGAIN! I'LL TEAR"
#define TXT_DIALOG7_LINE3								"THE FLESH FROM"
#define TXT_DIALOG7_LINE4								"YOUR BONES!!!"
#define TXT_DIALOG7_LINE5								"RRRAAAWLL!.."
#define TXT_DIALOG7_LINE6								"Wait.. You're"
#define TXT_DIALOG7_LINE7								"not her..."
#define TXT_DIALOG7_LINE8								"another tar"
#define TXT_DIALOG7_LINE9								"blob? rrrrrr.."

#define TXT_DIALOG8_LINE1								"Either way I'll"
#define TXT_DIALOG8_LINE2								"teach you to"
#define TXT_DIALOG8_LINE3								"disturb my"
#define TXT_DIALOG8_LINE4								"sleep!"
#define TXT_DIALOG8_LINE5								"Prepare to"
#define TXT_DIALOG8_LINE6								"be shredded!"

#define TXT_DIALOG9_LINE1								"Ah we meet again"
#define TXT_DIALOG9_LINE2								"little one...."
#define TXT_DIALOG9_LINE3								"Yess, again...."
#define TXT_DIALOG9_LINE4								"Do you enjoy our"
#define TXT_DIALOG9_LINE5								"clever disguises"
#define TXT_DIALOG9_LINE6								"little Gish?  I"
#define TXT_DIALOG9_LINE7								"think we blend in"
#define TXT_DIALOG9_LINE8								"quite well..Yess,"
#define TXT_DIALOG9_LINE9								"quite well..."

#define TXT_DIALOG10_LINE1							"BAH! Enough talk"
#define TXT_DIALOG10_LINE2							"small fry, this"
#define TXT_DIALOG10_LINE3							"time it won't be"
#define TXT_DIALOG10_LINE4							"so easy..Yess, no"
#define TXT_DIALOG10_LINE5							"happy for you!!"

#define TXT_DIALOG11_LINE1							"Hello Gish"
#define TXT_DIALOG11_LINE2							"What, you don't"
#define TXT_DIALOG11_LINE3							"remember me? We"
#define TXT_DIALOG11_LINE4							"went to the same"
#define TXT_DIALOG11_LINE5							"highschool, I sat"
#define TXT_DIALOG11_LINE6							"behind you in"
#define TXT_DIALOG11_LINE7							"math class, I lived"
#define TXT_DIALOG11_LINE8							"next door to you"
#define TXT_DIALOG11_LINE9							"for 6 years!  HOW"
#define TXT_DIALOG11_LINE10							"CAN YOU NOT KNOW"
#define TXT_DIALOG11_LINE11							"WHO I AM?"
#define TXT_DIALOG11_LINE12							"Everyday I see"
#define TXT_DIALOG11_LINE13							"you with HER!"
#define TXT_DIALOG11_LINE14							"Its...its just"
#define TXT_DIALOG11_LINE15							"not right...you"
#define TXT_DIALOG11_LINE16							"should be MINE, "
#define TXT_DIALOG11_LINE17							"she could never"
#define TXT_DIALOG11_LINE18							"love you like I do"
#define TXT_DIALOG11_LINE19							"GISH! WE were meant"
#define TXT_DIALOG11_LINE20							"to be together!"
#define TXT_DIALOG11_LINE21							"HERA+GISH=TRUE LOVE!"
#define TXT_DIALOG11_LINE22							"MARRY ME GISH,"
#define TXT_DIALOG11_LINE23							"MARRY ME NOW IN"
#define TXT_DIALOG11_LINE24							"THIS CHURCH OR YOUR"
#define TXT_DIALOG11_LINE25							"LITTLE TRAMP OF A"
#define TXT_DIALOG11_LINE26							"GIRL FRIEND WILL"
#define TXT_DIALOG11_LINE27							"NEVER LIVE TO SEE"
#define TXT_DIALOG11_LINE28							"THE LIGHT OF DAY!"
#define TXT_DIALOG11_LINE29             "..."
#define TXT_DIALOG11_LINE30							"WHAT DO YOU MEAN NO!"
#define TXT_DIALOG11_LINE31							"YOU LOSER, HOW DARE"
#define TXT_DIALOG11_LINE32							"YOU REJECT ME!  IF"
#define TXT_DIALOG11_LINE33							"I CAN'T HAVE YOU..."
#define TXT_DIALOG11_LINE34							"NO ONE CAN!"

#define TXT_DIALOG12_LINE1							"I AM SATAN!!!"
#define TXT_DIALOG12_LINE2							"You have proven your"
#define TXT_DIALOG12_LINE3							"worthiness to me, join"
#define TXT_DIALOG12_LINE4							"me in my quest to rule"
#define TXT_DIALOG12_LINE5							"the world and I shall"
#define TXT_DIALOG12_LINE6							"grant you eternal life."
#define TXT_DIALOG12_LINE7							"Join Satan?"
#define TXT_DIALOG12_LINE8							"*OK"
#define TXT_DIALOG12_LINE9							" Never"
#define TXT_DIALOG12_LINE10							"Join Satan?"
#define TXT_DIALOG12_LINE11							" OK"
#define TXT_DIALOG12_LINE12							"*Never"
#define TXT_DIALOG12_LINE13							"OK cool, I don't really"
#define TXT_DIALOG12_LINE14							"have a world domination"
#define TXT_DIALOG12_LINE15							"plan right now, you"
#define TXT_DIALOG12_LINE16							"have any ideas?"
#define TXT_DIALOG12_LINE17							"No?  Allright, give me"
#define TXT_DIALOG12_LINE18							"some time to come up"
#define TXT_DIALOG12_LINE19							"with something..."
#define TXT_DIALOG12_LINE20							"Damn Me, I'm not much of"
#define TXT_DIALOG12_LINE21							"an idea guy..."
#define TXT_DIALOG12_LINE22							"I gotta go"
#define TXT_DIALOG12_LINE23							"You sure you don't want"
#define TXT_DIALOG12_LINE24							"to hang out?  I have a"
#define TXT_DIALOG12_LINE25							"bunch of DVDs, a plasma"
#define TXT_DIALOG12_LINE26							"screen TV, Triptych..."
#define TXT_DIALOG12_LINE27							"OK, fine, whatever dude,"
#define TXT_DIALOG12_LINE28							"all I gave you is eternal"
#define TXT_DIALOG12_LINE29							"life."
#define TXT_DIALOG12_LINE30							"Very well then,"
#define TXT_DIALOG12_LINE31							"PREPARE TO DIE!!!"

#define TXT_GETREADY										"Get Ready"
#define TXT_TIE													"Its A Tie!"

#define TXT_TUTORIAL_LINE1							"Press the '/s' key to slide down"
#define TXT_TUTORIAL_LINE2							"the hole."
#define TXT_TUTORIAL_LINE3							"Switches and buttons sometimes"
#define TXT_TUTORIAL_LINE4							"open doors."
#define TXT_TUTORIAL_LINE5							"Press the '/s' key to stick to"
#define TXT_TUTORIAL_LINE6							"the wall.  Press the '/s' key"
#define TXT_TUTORIAL_LINE7							"to begin climbing"
#define TXT_TUTORIAL_LINE8							"Press the '/s' key to jump."
#define TXT_TUTORIAL_LINE9							"Gish jumps best when he is"
#define TXT_TUTORIAL_LINE10							"the most compressed, bouncing"
#define TXT_TUTORIAL_LINE11							"up and down will help you get"
#define TXT_TUTORIAL_LINE12							"higher and higher."
#define TXT_TUTORIAL_LINE13							"Press the '/s' key to break"
#define TXT_TUTORIAL_LINE14							"these blocks."
#define TXT_TUTORIAL_LINE15							"Press the '/s' key to stick to"
#define TXT_TUTORIAL_LINE16							"the wall.  Press the '/s' and"
#define TXT_TUTORIAL_LINE17							"'/s' keys to climb. If you are"
#define TXT_TUTORIAL_LINE18							"stuck to the floor, quickly release"
#define TXT_TUTORIAL_LINE19							"and then press the stick key"
#define TXT_TUTORIAL_MOVEMENT						"Movement"
#define TXT_TUTORIAL_STICK							"Stick"
#define TXT_TUTORIAL_JUMP								"Jump"
#define TXT_TUTORIAL_SLIDE							"Slide"
#define TXT_TUTORIAL_HEAVY							"Heavy"

//mainmenu.c
#define TXT_LOADING											"Loading..."
#define TXT_TURBO_EDITION								"Turbo Edition"
#define TXT_COPYRIGHT                   "Gish (c) 2007 Cryptic Sea"
#define TXT_EXIT												"Exit"
#define TXT_START												"Start"
#define TXT_VERSUS											"Versus"
#define TXT_OPTIONS											"Options"
#define TXT_CREDITS											"Credits"
#define TXT_BUYNOW											"Buy Now!"
#define TXT_BACK												"Back"
#define TXT_SUMO												"Sumo"
#define TXT_FOOTBALL										"Football"
#define TXT_GREED												"Greed"
#define TXT_DUEL										"Duel"
#define TXT_DRAGSTER										"Dragster"
#define TXT_COLLECTION									"Collection"
#define TXT_RACING											"Racing"
#define TXT_NEXT												"Next"

//credits.c
#define TXT_DESIGN											"Design"
#define TXT_PROGRAMMING									"Programming"
#define TXT_ART													"Art"
#define TXT_SOUND_MUSIC									"Sound//Music"
#define TXT_ADDITIONAL_DIALOG						"Additional Dialog & Sounds"
#define TXT_ADDITIONAL_CONTENT					"Additional Level Design & Art"
#define TXT_THANKS											"Thanks"
#define TXT_GISHUSES										"Gish uses"

//editor.c
#define TXT_LEVELNAME										"Level Name"
#define TXT_GAMETYPE										"Gametype"
#define TXT_GAMETIME										"Gametime"
#define TXT_RED													"Red"
#define TXT_GREEN												"Green"
#define TXT_BLUE												"Blue"
#define TXT_BACKGROUND									"Background"
#define TXT_MIDGROUND										"MIDGROUND"
#define TXT_FOREGROUND									"FOREGROUND"
#define TXT_WAYBACKGROUND								"WAYBACKGROUND"
#define TXT_TILESET											"TILESET"
#define TXT_TILE												"TILE"
#define TXT_FRICTION										"Friction"
#define TXT_BREAKPOINT									"Breakpoint"
#define TXT_MIDDAMAGE										"Middamage"
#define TXT_FOREDAMAGE									"Foredamage"
#define TXT_DENSITY											"Density"
#define TXT_DRAG												"Drag"
#define TXT_ANIMATION										"Animation"
#define TXT_ANIMATESPD									"AnimateSpd"
#define TXT_LINES												"Lines"
#define TXT_ALPHA												"Alpha"
//game.c
#define TXT_REPLAY											"Replay"
#define TXT_PAUSED											"Paused"
#define TXT_PRESS_P											"Press P to continue"
#define TXT_LOADINGEDITOR								"Loading Editor"
#define TXT_RETURN_TO_GAME							"Return To Game"
#define TXT_RESETLEVEL_MINUSONE					"Restart Level (-1 Life)"
#define TXT_RESETLEVEL									"Restart Level"
#define TXT_EXITGAME_MINUSONE						"Exit Game (-1 Life)"
#define TXT_EXITGAME										"Exit Game"
#define TXT_TIMEUP											"TIME UP!"
#define TXT_COMPLETE										"COMPLETE"
#define TXT_WARPZONE										"WARP ZONE!"
#define TXT_OOPS												"OOPS!"
//gamemenu.c
#define TXT_NEXTLEVEL										"Next Level"
#define TXT_ENDING											"Ending"
#define TXT_SMALL_AMBER									"Small Amber"
#define TXT_MEDIUM_AMBER								"Medium Amber"
#define TXT_LARGE_AMBER									"Large Amber"
#define TXT_TARBALLS										"Tarballs"
#define TXT_EXTRA_LIVES									"Extra Lives"
#define TXT_SECRETS											"Secrets"
#define TXT_LIFE_BONUS									"Life Bonus"
#define TXT_LEVEL_POINTS								"Level Points"
#define TXT_GF_RESCUING_BONUS						"Girlfriend Rescuing Bonus:1"
#define TXT_BOSS_POINTS									"Boss Points"
#define TXT_TOTAL_POINTS								"Total Points"
#define TXT_LIVES												"Lives"
#define TXT_POINTS											"Points"
#define TXT_ISLE_OF_THE									"Isle of the"
#define TXT_DEAD												"Dead"
#define TXT_SATANS_LAIR									"Satan's Lair"
#define TXT_CONTINUE										"Continue"
#define TXT_GAME_OVER										"GAME OVER"

#define TXT_ENDING_LINE1								"After rescuing Brea from the"
#define TXT_ENDING_LINE2								"clutches of that obsessed psycho"
#define TXT_ENDING_LINE3								"Hera, Gish and Brea went on to"
#define TXT_ENDING_LINE4								"become world renowned "
#define TXT_ENDING_LINE5								"Entomologists and had the world's"
#define TXT_ENDING_LINE6								"first legal inter-species marriage"
#define TXT_ENDING_LINE7								"Hera was never heard from again,"
#define TXT_ENDING_LINE8								"though some say they can hear her"
#define TXT_ENDING_LINE9								"calling Gish's name from the"
#define TXT_ENDING_LINE10								"depths of Gehenna.  Sadly, to"
#define TXT_ENDING_LINE11								"this day no one seems to remember"
#define TXT_ENDING_LINE12								"who she was..."

#define TXT_ENDING_BAD_LINE1						"After letting Brea fall into the"
#define TXT_ENDING_BAD_LINE2						"pit of lava, Gish went on to lead"
#define TXT_ENDING_BAD_LINE3						"a life of celibacy, volunteering"
#define TXT_ENDING_BAD_LINE4						"most of his time to charity"
#define TXT_ENDING_BAD_LINE5						"organizations that specialize in"
#define TXT_ENDING_BAD_LINE6						"bringing lava awareness to the"
#define TXT_ENDING_BAD_LINE7						"mainstream."
#define TXT_ENDING_BAD_LINE8						"Hera was never heard from again,"
#define TXT_ENDING_BAD_LINE9						"though some say they can hear her"
#define TXT_ENDING_BAD_LINE10						"calling Gish's name from the"
#define TXT_ENDING_BAD_LINE11						"depths of Gehenna.  Sadly, to"
#define TXT_ENDING_BAD_LINE12						"this day no one seems to remember"
#define TXT_ENDING_BAD_LINE13						"who she was..."

#define TXT_NEW_VS_MODE									"New Vs. Mode Unlocked"
#define TXT_NEW_VS_LEVEL								"New Vs. Levels Unlocked"
#define TXT_MINI_GISH_MODE							"Mini Gish Mode Unlocked"

//high.c
#define TXT_SAVEREPLAY									"Save Replay"
#define TXT_LEVEL 											"Level"
#define TXT_LEVEL_HIGHSCORE							"Level /i High Scores"
#define TXT_NAME												"Name"
#define TXT_TIME												"Time"
#define TXT_REPLAYSAVED									"Replay Saved"
//objedit.c
#define TXT_OBJECTSET										"OBJECTSET"
#define TXT_OBJECTNUM										"OBJECTNUM"
#define TXT_OBJECTYPE										"OBJECTTYPE"
#define TXT_OBJECTS											"OBJECTS"
#define TXT_ROPES												"ROPES"
#define TXT_LINK												"LINK"
//options.c
#define TXT_VIDEOOPTIONS								"Video Options"
#define TXT_NONE												"None"
#define TXT_JOY1												"Joy1"
#define TXT_JOY2												"Joy2"
#define TXT_JOY3												"Joy3"
#define TXT_JOY4												"Joy4"
#define TXT_SOUND_ON										"Sound On"
#define TXT_SOUND_OFF										"Sound Off"
#define TXT_MUSIC_ON										"Music On"
#define TXT_MUSIC_OFF										"Music Off"
#define TXT_PLAYER											"Player"
#define TXT_KEY													"Key"
#define TXT_JOYSTICK										"Joystick"
#define TXT_PLAYER1											"Player 1"
#define TXT_MOVE_LEFT										"Move Left"
#define TXT_BUTTON											"Button"
#define TXT_AXIS												"Axis"
#define TXT_MOVE_RIGHT									"Move Right"
#define TXT_MOVE_DOWN										"Move Down"
#define TXT_MOVE_UP											"Move Up"
#define TXT_STICK												"Stick"
#define TXT_START_PAUSE									"Start//Pause"
#define TXT_PLAYER2											"Player 2"
#define TXT_APPLY												"Apply"
#define TXT_FULLSCREEN									"Fullscreen"
#define TXT_16BIT												"16 bit"
#define TXT_32BIT												"32 bit"
#define TXT_RESOLUTION									"Resolution"
#define TXT_COLOR												"Color"
#define TXT_OPENGLINFO									"OpenGL Info"
#define TXT_JUMP												"Jump"
#define TXT_STICK												"Stick"
#define TXT_SLIDE												"Slide"
#define TXT_HEAVY												"Heavy"

//player.c
#define TXT_DELETE_PLAYER								"Delete Player"
#define TXT_CREATE											"Create"
#define TXT_PLAYER_INFO									"Level /i-/i Lives /i Score /i High /i"
#define	TXT_NEW_PLAYER									"/i-New Player"																
#define TXT_CHOOSE_PLAYER								"Choose Player"
#define TXT_ENTER_NAME									"Enter Name"
#define TXT_DELETE_Y_N									"Delete /s? (Y//N)"
#define TXT_COLLECTION_GAME							"Collection Game"
#define TXT_NEW_GAME										"New Game"
#define TXT_PRACTICE                    "Practice"
#define TXT_SINGLE_LEVEL								"Single Level"
#define TXT_NEW_LEVEL										"New Level"
#define TXT_EASY												"Easy" 
#define TXT_NORMAL											"Normal"
#define TXT_HARD												"Hard"
#define TXT_LUDICROUS										"Ludicrous"
#define TXT_MINI_GISH										"Mini Gish"
#define TXT_HIGH_SCORE									"High Scores"
#define TXT_COLLECTION_MODE							"Collection Mode"
#define TXT_SELECT_DIFFICULTY						"Select Difficulty"

//replay.c
#define TXT_PLAY_REPLAY									"Play Replay"
#define TXT_FILENAME										"Filename"
#define TXT_REPLAY_MODE									"Replay Mode"

//ropeedit.c
#define TXT_PLAY_REPLAY									"Play Replay"
#define TXT_NUMOFROPES									"NUMOFROPES"
#define TXT_TEXTURENUM									"TEXTURENUM"

//texture.c
#define TXT_LOAD_TILESET								"Load Tileset"
#define TXT_LEVEL_NAME									"Level Name"

#define TXT_PLAYER3                     "Player 3"
#define TXT_PLAYER4                     "Player 4"
#define TXT_4_PLAYER                    "4 Player"
#define TXT_CANCEL                      "Cancel"
#define TXT_CUSTOM_LEVELS               "Custom Levels"
#define TXT_LOAD_LEVEL                  "Load Level"
#define TXT_PAGE_UP                     "Page Up"
#define TXT_PAGE_DOWN                   "Page Down"
#define TXT_PLAY_CAMPAIGN               "Play Campaign"
#define TXT_PLAYER_INFO_MAPPACK         "Level /s Lives /i Score /i High /i"
#define TXT_SAVE                        "Save"
#define TXT_CAMPAIGN_EDITOR             "Campaign Editor"
#define TXT_LEVEL_EDITOR                "Level Editor"
#define TXT_ADD_LEVEL                   "Add Level"
#define TXT_DELETE_LEVEL                "Delete Level"

#endif /* GISH_GAME_ENGLISH_H */
